@@ -1,4 +1,4 @@
-"""Declarative base class/factory and query property support.
+"""Declarative base for ORM models.
 """
 
 from sqlalchemy import inspect, orm
@@ -6,16 +6,29 @@ from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 
 from . import query, events
 from .utils import (
-    classproperty, is_sequence, has_primary_key, camelcase_to_underscore)
+    is_sequence,
+    has_primary_key,
+    camelcase_to_underscore
+)
 from ._compat import iteritems
 
 
+__all__ = [
+    'ModelMeta',
+    'ModelBase',
+    'make_declarative_base',
+    'extend_declarative_base',
+]
+
+
 class ModelMeta(DeclarativeMeta):
     """ModelBase's metaclass which provides:
 
-        - Tablename autogeneration
-        - Multiple database binding
-        - Declarative ORM events
+    - Tablename autogeneration when ``__tablename__`` not defined.
+    - Support for multiple database bindings via \
+    :attr:`ModelBase.__bind_key__`.
+    - Support for declarative ORM events via :mod:`alchy.events` decorators \
+    or :attr:`ModelBase.__events__`.
     """
     def __new__(mcs, name, bases, dct):
         # Determine if primary key is defined for dct or any of its bases.
@@ -49,42 +62,82 @@ class ModelMeta(DeclarativeMeta):
 
 
 class ModelBase(object):
-    """Augmentable Base class for adding shared model properties/functions"""
+    """Base class for creating a declarative base for models.
+
+    To create a declarative base::
+
+        # in project/core.py
+        from alchy import ModelBase, make_declarative_base
+
+        class Base(ModelBase):
+            # augument the ModelBase with super powers
+            pass
+
+        Model = make_declarative_base(Base=Base)
+
+
+        # in project/models/user.py
+        from project.core import Model
+
+        class User(Model):
+            # define declarative User model
+            pass
+
+
+    Attributes:
+
+        __table_args__: Default table args.
+
+        __mapper_args__: Define a default order by when not specified by query
+            operation, e.g.: ``{ 'order_by': [column1, column2] }``
+
+        __bind_key__: Bind a model to a particular database URI using keys from
+            ``Manager.config['SQLALCHEMY_BINDS']``. By default a model will be
+            bound to ``Manager.config['SQLALCHEMY_DATABASE_URI']``.
+
+        __events__: Register orm event listeners. See :mod:`alchy.events` for
+            more details.
+
+        query_class: Query class to use for ``self.query``.
+
+        query: An instance of :attr:`query_class`. Can be used to query the
+            database for instances of this model. NOTE: Requires setting
+            ``MyClass.query = QueryProperty(session)`` when session available.
+            See :func:`make_declarative_base` for automatic implementation.
+    """
 
-    # default table args
     __table_args__ = {}
-
-    # define a default order by when not specified by query operation
-    # eg: { 'order_by': [column1, column2] }
     __mapper_args__ = {}
-
-    # register orm event listeners
+    __bind_key__ = None
     __events__ = {}
 
-    # query class to use for self.query
     query_class = query.QueryModel
-
-    # An instance of `query_class`.
-    # Can be used to query the database for instances of this model
-    # Note: requires setting Class.query = QueryProperty(session) when session
-    # available. See `make_declarative_base()` for automatic implementation.
     query = None
 
     def __init__(self, *args, **kargs):
+        """Initialize model instance by calling :meth:`update`."""
         self.update(*args, **kargs)
 
     def __repr__(self):  # pragma: no cover
         values = ', '.join(['{0}={1}'.format(c, repr(getattr(self, c)))
-                            for c in self.columns])
+                            for c in self.columns()])
         return '<{0}({1})>'.format(self.__class__.__name__, values)
 
+    def __getitem__(self, item):
+        """Proxy getitem to getattr."""
+        return getattr(self, item)
+
+    def __setitem__(self, item, value):
+        """Proxy setitem to setattr."""
+        setattr(self, item, value)
+
     def update(self, data_dict=None, **kargs):
         """Update model with arbitrary set of data."""
 
         data = data_dict if isinstance(data_dict, dict) else kargs
 
         update_fields = data.keys()
-        relationships = self.relationships
+        relationships = self.relationships()
 
         for field, value in iteritems(data):
             if hasattr(self, field) and field in update_fields:
@@ -111,47 +164,47 @@ class ModelBase(object):
 
     @property
     def __to_dict__(self):
-        """Configuration for `to_dict()`. Do any necessary preprocessing and
-        return a set of string attributes which represent the fields which
-        should be returned when calling `to_dict()`.
+        """Configuration for :meth:`to_dict`. Do any necessary preprocessing
+        and return a set of string attributes which represent the fields which
+        should be returned when calling :meth:`to_dict`.
 
-        By default this model is refreshed if it's __dict__ state is empty and
-        only the ORM descriptor fields are returned.
+        By default this model is refreshed if it's ``__dict__`` state is empty
+        and only the ORM descriptor fields are returned.
 
         This is the property to override if you want to return more/less than
         the default ORM descriptor fields.
 
-        Generally, we can usually rely on `self.__dict__` as a representation
-        of model when it's just been loaded from the database. In this case,
-        whatever values are present in `__dict__` are the loaded values from
-        the database which include/exclude lazy attributes (columns and
+        Generally, we can usually rely on ``__dict__`` as a representation of
+        model when it's just been loaded from the database. In this case,
+        whatever values are present in ``__dict__`` are the loaded values
+        from the database which include/exclude lazy attributes (columns and
         relationships).
 
         One issue to be aware of is that after a model has been committed (or
-        expired), __dict__ will be empty. This can be worked around by calling
-        `self.refresh()` which will reload the data from the database using the
-        default loader strategies.
+        expired), ``__dict__`` will be empty. This can be worked around by
+        calling :meth:`refresh` which will reload the data from the database
+        using the default loader strategies.
 
         These are the two main cases this default implementation will try to
         cover. For anything more complex it would be best to override this
-        property or the `to_dict()` method itself.
+        property or the :meth:`to_dict` method itself.
         """
         if not self.descriptor_dict.keys() and orm.object_session(self):
-            # if the descriptor dict keys are empty, assume we need to refresh
+            # If the descriptor dict keys are empty, assume we need to refresh.
             self.refresh()
 
         return set(self.descriptor_dict.keys())
 
     @property
     def descriptor_dict(self):
-        """Return `self.__dict__` key-filtered by `self.descriptors`"""
+        """Return :attr:`__dict__` key-filtered by :attr:`descriptors`."""
         return dict([(key, value)
                      for key, value in iteritems(self.__dict__)
-                     if key in self.descriptors])
+                     if key in self.descriptors()])
 
     def to_dict(self):
         """Return dict representation of model by filtering fields using
-        `self.__to_dict__`.
+        :attr:`__to_dict__`.
         """
         data = {}
         data_fields = self.__to_dict__
@@ -175,7 +228,9 @@ class ModelBase(object):
         return data
 
     def __iter__(self):
-        """Implement __iter__ so model can be converted to dict via dict()."""
+        """Implement ``__iter__`` so model can be converted to dict via
+        ``dict()``.
+        """
         return iteritems(self.to_dict())
 
     ##
@@ -184,54 +239,56 @@ class ModelBase(object):
 
     @property
     def object_session(self):
-        """Return session belonging to self"""
+        """Return session belonging to ``self``"""
         return orm.object_session(self)
 
-    @classproperty
+    @classmethod
     def session(cls):
         """Return session from query property"""
         return cls.query.session
 
     def flush(self, *args, **kargs):
-        """Call `session.flush` on `self`"""
-        self.session.flush([self], *args, **kargs)
+        """Call ``session.flush()`` on ``self``"""
+        self.session().flush([self], *args, **kargs)
 
     def save(self, *args, **kargs):
-        """Call `session.add` on `self`"""
-        self.session.add(self, *args, **kargs)
+        """Call ``session.add()`` on ``self``"""
+        self.session().add(self, *args, **kargs)
 
     def delete(self, *args, **kargs):
-        """Call `session.delete` on `self`"""
-        return self.session.delete(self, *args, **kargs)
+        """Call ``session.delete()`` on ``self``"""
+        return self.session().delete(self, *args, **kargs)
 
     def expire(self, *args, **kargs):
-        """Call `session.expire` on `self`"""
-        self.session.expire(self, *args, **kargs)
+        """Call ``session.expire()`` on ``self``"""
+        self.session().expire(self, *args, **kargs)
 
     def refresh(self, *args, **kargs):
-        """Call `session.refresh` on `self`"""
-        self.session.refresh(self, *args, **kargs)
+        """Call ``session.refresh()`` on ``self``"""
+        self.session().refresh(self, *args, **kargs)
 
     def expunge(self, *args, **kargs):
-        """Call `session.expunge` on `self`"""
-        self.session.expunge(self, *args, **kargs)
+        """Call ``session.expunge()`` on ``self``"""
+        self.session().expunge(self, *args, **kargs)
 
     @classmethod
     def get(cls, *args, **kargs):
-        """Proxy to query.get()"""
+        """Proxy to ``cls.query.get()``"""
         return cls.query.get(*args, **kargs)
 
     @classmethod
     def get_by(cls, data_dict=None, **kargs):
-        """Return first instance filtered by values using query.filter_by()"""
+        """Return first instance filtered by values using
+        ``cls.query.filter_by()``.
+        """
         data = data_dict if isinstance(data_dict, dict) else kargs
         return cls.query.filter_by(**data).first()
 
     ##
-    # inspect based methods/properties
+    # SQLAlchemy.inspect() based methods/properties
     ##
 
-    @classproperty
+    @classmethod
     def primary_key(cls):
         """Return primary key as either single column (one primary key) or
         tuple otherwise.
@@ -243,44 +300,44 @@ class ModelBase(object):
 
         return primary
 
-    @classproperty
+    @classmethod
     def primary_keys(cls):
         """Return primary keys as tuple."""
         return inspect(cls).primary_key
 
-    @classproperty
+    @classmethod
     def primary_attrs(cls):
         """Return class attributes from primary keys."""
-        primary_keys = cls.primary_keys
+        primary_keys = cls.primary_keys()
         return [getattr(cls, attr)
-                for attr in cls.columns
+                for attr in cls.columns()
                 if getattr(cls, attr).property.columns[0] in primary_keys]
 
-    @classproperty
+    @classmethod
     def attrs(cls):
         """Return ORM attributes"""
         return inspect(cls).attrs.keys()
 
-    @classproperty
+    @classmethod
     def descriptors(cls):
         """Return all ORM descriptors"""
         return [descr for descr in inspect(cls).all_orm_descriptors.keys()
                 if not descr.startswith('__')]
 
-    @classproperty
+    @classmethod
     def relationships(cls):
         """Return ORM relationships"""
         return inspect(cls).relationships.keys()
 
-    @classproperty
+    @classmethod
     def column_attrs(cls):
         """Return table columns as list of class attributes at the class level.
         """
         return inspect(cls).column_attrs
 
-    @classproperty
+    @classmethod
     def columns(cls):
-        """Return table columns"""
+        """Return table columns."""
         return inspect(cls).columns.keys()
 
 
@@ -290,8 +347,9 @@ def make_declarative_base(session=None, Model=None, Base=None):
     """
     if Model is None:
         Base = Base or ModelBase
-        Model = declarative_base(
-            cls=Base, constructor=Base.__init__, metaclass=ModelMeta)
+        Model = declarative_base(cls=Base,
+                                 constructor=Base.__init__,
+                                 metaclass=ModelMeta)
 
     extend_declarative_base(Model, session)
 
